from sklearn.linear_model import LinearRegression
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt
import numpy as np

n_cells = np.reshape(np.array([
    10547,
    26369,
    52738,
    105476
]), (4, 1))

pano_memory = np.reshape(np.array([
    1.1,
    2.8,
    6.1,
    12.9
]), (4, 1))

cca_memory = np.reshape(np.array([
    5.0,
    12.9,
    26.1,
    54.2
]), (4, 1))

mnn_memory = np.reshape(np.array([
    5.1,
    13.1,
    25.7,
    49.5
]), (4, 1))

pano_runtime = np.reshape(np.array([
    40.9,
    110.1,
    204.8,
    469.6,
]) / 3600., (4, 1))

cca_runtime = np.reshape(np.array([
    9724.1,
    24622.1,
    49243.7,
    99683.4,
]) / 3600., (4, 1))

mnn_runtime = np.reshape(np.array([
    15669.2,
    39899.7,
    78677.3,
    157212.6
]) / 3600., (4, 1))

line_x = np.array(range(n_cells[-1]), dtype=int)
line_x = line_x.reshape(-1, 1)

# Memory plot.
plt.figure()
plt.plot(line_x, LinearRegression().fit(n_cells, pano_memory)
         .predict(line_x), 'k')
pano = plt.scatter(n_cells, pano_memory, marker='o')
plt.plot(line_x, LinearRegression().fit(n_cells, cca_memory)
         .predict(line_x), 'k')
cca = plt.scatter(n_cells, cca_memory, marker='^')
plt.plot(line_x, LinearRegression().fit(n_cells, mnn_memory)
         .predict(line_x), 'k')
mnn = plt.scatter(n_cells, mnn_memory, marker='s')
plt.legend((pano, cca, mnn),
           ('Scanorama', 'Seurat CCA', 'scran MNN'))
plt.xlabel('Number of cells')
plt.ylabel('Memory (GB)')
plt.savefig('benchmark_memory.svg')

# Memory plot.
plt.figure()
plt.plot(line_x, LinearRegression().fit(n_cells, pano_runtime)
         .predict(line_x), 'k')
pano = plt.scatter(n_cells, pano_runtime, marker='o')
plt.plot(line_x, LinearRegression().fit(n_cells, cca_runtime)
         .predict(line_x), 'k')
cca = plt.scatter(n_cells, cca_runtime, marker='^')
plt.plot(line_x, LinearRegression().fit(n_cells, mnn_runtime)
         .predict(line_x), 'k')
mnn = plt.scatter(n_cells, mnn_runtime, marker='s')
plt.legend((pano, cca, mnn),
           ('Scanorama', 'Seurat CCA', 'scran MNN'))
plt.yscale('log')
plt.xlabel('Number of cells')
plt.ylabel('Runtime (hours)')
plt.savefig('benchmark_runtime.svg')



import numpy as np
from scanorama import *
from sklearn.preprocessing import normalize, LabelEncoder
import sys

from process import load_names, merge_datasets, save_datasets

NAMESPACE = '293t_jurkat'

data_names = [
    'data/293t_jurkat/293t',
    'data/293t_jurkat/jurkat',
    'data/293t_jurkat/jurkat_293t_50_50',
]

if __name__ == '__main__':
    datasets, genes_list, n_cells = load_names(data_names)

    datasets_dimred, datasets, genes = correct(
        datasets, genes_list, ds_names=data_names,
        return_dimred=True
    )

    save_datasets(datasets, genes, data_names)

    labels = []
    names = []
    curr_label = 0
    for i, a in enumerate(datasets):
        labels += list(np.zeros(a.shape[0]) + curr_label)
        names.append(data_names[i])
        curr_label += 1
    labels = np.array(labels, dtype=int)
    
    embedding = visualize(datasets_dimred,
                          labels, NAMESPACE + '_ds', names,
                          perplexity=600, n_iter=400, size=100)
    
    cell_labels = (
        open('data/cell_labels/293t_jurkat_cluster.txt')
        .read().rstrip().split()
    )
    le = LabelEncoder().fit(cell_labels)
    labels = le.transform(cell_labels)
    cell_types = le.classes_
    
    visualize(None,
              labels, NAMESPACE + '_type', cell_types,
              perplexity=600, n_iter=400, size=100,
              embedding=embedding) 

    # Uncorrected.
    datasets, genes_list, n_cells = load_names(data_names)
    datasets, genes = merge_datasets(datasets, genes_list)
    datasets = [ normalize(ds, axis=1) for ds in datasets ]
    datasets_dimred = dimensionality_reduce(datasets)
    
    visualize(datasets_dimred, labels,
              NAMESPACE + '_type_uncorrected', cell_types,
              perplexity=600, n_iter=400, size=100)



import fileinput
import sys

data_names = None

if data_names is None:
    if len(sys.argv) == 1:
        print('Enter data names followed by EOF/Ctrl-D:')
    
    data_names = []
    for line in fileinput.input():
        fields = line.rstrip().split(',')
        for f in fields:
            if f.strip() == '':
                continue
            data_names.append(f)
    print('Data names loaded')



import numpy as np
from scanorama import *
from scipy.stats import f_oneway
from sklearn.cluster import KMeans
import sys

def print_oneway(X, genes, ds_labels):
    for gene_idx, gene in enumerate(genes):
        ds_names = sorted(set(ds_labels))
        dist = []
        for ds in ds_names:
            dist.append(X[ds_labels == ds, gene_idx])
        sys.stdout.write('{}\t'.format(gene))
        print('{}\t{}'.format(*f_oneway(*dist)))
        
def entropy_test(datasets_dimred, ds_labels):
    
    ds_labels = np.array(ds_labels)
    X_dimred = np.concatenate(datasets_dimred)
    embedding = None
    
    for k in range(10, 21):
        km = KMeans(n_clusters=k, n_jobs=-1, verbose=0)
        km.fit(X_dimred)

        if False and k % 5 == 0:
            embedding = visualize(
                datasets_dimred,
                km.labels_, NAMESPACE + '_km{}'.format(k),
                [ str(x) for x in range(k) ],
                embedding=embedding
            )
        
        print('k = {}, average normalized entropy = {}'
              .format(k, avg_norm_entropy(ds_labels, km.labels_)))

def avg_norm_entropy(ds_labels, cluster_labels):
    assert(len(ds_labels) == len(cluster_labels))
    
    clusters = sorted(set(cluster_labels))
    datasets = sorted(set(ds_labels))

    Hs = []
    for cluster in clusters:

        cluster_idx = cluster_labels == cluster
        ds_rep = ds_labels[cluster_idx]
        n_cluster = float(sum(cluster_idx))

        H = 0
        for ds in datasets:
            n_ds = float(sum(ds_rep == ds))
            if n_ds == 0: # 0 log 0 = 0
                continue
            H += (n_ds / n_cluster) * np.log(n_ds / n_cluster)
        H *= -1
        H /= np.log(len(datasets))

        Hs.append(H)
        
    return np.mean(Hs)



import numpy as np
from scanorama import *
from sklearn.metrics import silhouette_samples as sil
from scipy.sparse import vstack
from sklearn.preprocessing import normalize, LabelEncoder
import sys

from pancreas_tests import *
from process import load_names

NAMESPACE = 'pancreas'

data_names = [
    'data/pancreas/pancreas_inDrop',
    'data/pancreas/pancreas_multi_celseq2_expression_matrix',
    'data/pancreas/pancreas_multi_celseq_expression_matrix',
    'data/pancreas/pancreas_multi_fluidigmc1_expression_matrix',
    'data/pancreas/pancreas_multi_smartseq2_expression_matrix',
]

if __name__ == '__main__':
    datasets, genes_list, n_cells = load_names(data_names)
    
    labels = []
    names = []
    curr_label = 0
    for i, a in enumerate(datasets):
        labels += list(np.zeros(a.shape[0]) + curr_label)
        names.append(data_names[i])
        curr_label += 1
    labels = np.array(labels, dtype=int)

    datasets_dimred, datasets, genes = correct(
        datasets, genes_list, ds_names=data_names,
        return_dimred=True
    )
    
    cell_labels = (
        open('data/cell_labels/pancreas_cluster.txt')
        .read().rstrip().split()
    )
    le = LabelEncoder().fit(cell_labels)
    cell_labels = le.transform(cell_labels)
    cell_types = le.classes_
    
    print_oneway(vstack(datasets).toarray(), genes, labels)

    X_pan = vstack(datasets).toarray()
    var_pan = {
        genes[i]: val
        for i, val in enumerate(np.var(X_pan, axis=0))
    }

    pancreas_genes = [
        'HADH', 'G6PC2', 'PAPSS2', 'PCSK1', 'GC',
        'TTR', 'GCG', 'GPX3', 'VGF', 'CST3', 'KRT7',
        'ZFP36L1', 'KRT19',  'LAD1', 'FLNA', 'AHNAK',
        'ANXA2', 'RBP4', 'SST', 'FLT1', 'PLVAP', 'ENG',
        'S1PR1', 'EGFL7', 'CD93', 'ESM1', 'KDR', 'PPY',
        'BTG2', 'HERPUD1', 'GADD45A', 'LCN2', 'SDC4',
    ]

    embedding = visualize(datasets_dimred,
                          labels, NAMESPACE + '_ds', names,
                          gene_names=pancreas_genes, genes=genes,
                          gene_expr=vstack(datasets),
                          perplexity=100, n_iter=400)
    
    visualize(datasets_dimred,
              cell_labels, NAMESPACE + '_type', cell_types,
              embedding=embedding)

    # scran MNN.
    datasets, genes_list, n_cells = load_names(['data/mnn_corrected_pancreas'])
    
    datasets_dimred, datasets, genes = correct(
        datasets, genes_list, ds_names=data_names,
        return_dimred=True
    )
    genes = [ g.decode('utf-8') for g in genes ]
    
    print_oneway(vstack(datasets).toarray(), genes, labels)
    
    var_mnn = {
        genes[i]: val
        for i, val in enumerate(np.var(vstack(datasets).toarray(), axis=0))
    }
    
    # Uncorrected.
    datasets, genes_list, n_cells = load_names(data_names)
    datasets, genes = merge_datasets(datasets, genes_list)
    datasets = [ normalize(ds, axis=1) for ds in datasets ]
    datasets_dimred = dimensionality_reduce(datasets)

    print_oneway(vstack(datasets).toarray(), genes, labels)

    X_non = vstack(datasets).toarray()
    var_non = {
        genes[i]: val
        for i, val in enumerate(np.var(X_non, axis=0))
    }
    
    embedding = visualize(datasets_dimred, labels,
                          NAMESPACE + '_ds_uncorrected', names,
                          perplexity=100, n_iter=400)
    visualize(None, cell_labels,
              NAMESPACE + '_type_uncorrected', cell_types,
              embedding=embedding,
              perplexity=100, n_iter=400)

    from scipy.stats import pearsonr

    rhos = []
    assert(X_pan.shape == X_non.shape)
    for i in range(X_pan.shape[0]):
        rho, p = pearsonr(X_pan[i], X_non[i])
        rhos.append(rho)

    print(max(rhos))
    print(min(rhos))
    print(np.mean(rhos))
    print(np.median(rhos))
    print(np.var(rhos))
    
    in_common = sorted(set(var_pan.keys()) & set(var_non.keys()))
    print(pearsonr([ var_pan[g] for g in in_common ],
                    [ var_non[g] for g in in_common ]))
    in_common = sorted(set(var_mnn.keys()) & set(var_non.keys()))
    print(pearsonr([ var_mnn[g] for g in in_common ],
                    [ var_non[g] for g in in_common ]))




from process import load_names, merge_datasets

def write_table(dataset, genes, name, cell_name=None):
    if cell_name is not None:
        prefix = cell_name
    else:
        prefix = name.split('/')[-1]
    with open(name + '_table.txt', 'w') as f:
        header = '\t'.join([ prefix + str(i) for i in range(dataset.shape[0]) ])
        f.write(header + '\n')

        for i in range(dataset.shape[1]):
            line = '\t'.join([ str(j) for j in dataset[:, i] ])
            f.write(genes[i] + '\t' + line + '\n')

if __name__ == '__main__':
    from config import data_names
    
    # Load raw data from files.
    datasets_full, genes_list, n_cells = load_names(data_names)
    datasets, genes = merge_datasets(datasets_full, genes_list)

    for i in range(len(datasets)):
        print('Writing {}'.format(data_names[i]))
        write_table(datasets[i], genes, data_names[i])



import fileinput
from scanorama import plt
plt.rcParams.update({'font.size': 22})
import sys

methods = [
    'scanorama',
    'mnn',
    'seurat',
    'uncorrected'
]
data = {}

curr_method = 'scanorama'
ks = []
entropies = []

f = open(sys.argv[1])

for line in fileinput.input():
    line = line.rstrip()
    if line in methods:
        if len(ks) > 0:
            data[curr_method] = entropies
            ks = []
            entropies = []
        curr_method = line
        continue
    
    fields = line.split()

    k = int(fields[2].rstrip(','))
    if k < 10:
        continue
    
    ks.append(k)
    entropies.append(float(fields[-1]))
    
data[curr_method] = entropies

plt.figure()

for method in methods:
    plt.plot(ks, data[method], label=method)
    plt.scatter(ks, data[method])

plt.legend()
plt.xlabel('k-means, number of clusters')
plt.ylabel('Normalized Shannon entropy')
plt.savefig('entropies.svg')



import numpy as np
from scanorama import *
from scipy.sparse import vstack, csr_matrix
from sklearn.preprocessing import normalize, LabelEncoder
import sys

from benchmark import write_table
from process import load_names

NAMESPACE = 'simulate_rare'

data_names = [
    'data/simulation/simulate_rare/simulate_rare_0',
    'data/simulation/simulate_rare/simulate_rare_1',
    'data/simulation/simulate_rare/simulate_rare_2',
    'data/simulation/simulate_rare/simulate_rare_3',
    'data/simulation/simulate_rare/simulate_rare_4',
]

if __name__ == '__main__':
    datasets, genes_list, n_cells = load_names(data_names)
    n_genes = len(genes_list[0])
    
    #for i in range(len(datasets)):
    #    print('Writing {}'.format(data_names[i]))
    #    write_table(datasets[i].toarray(), genes_list[i], data_names[i])
    #exit()

    datasets_dimred, _, genes = correct(
        datasets[:], genes_list, ds_names=data_names,
        return_dimred=True
    )

    labels = []
    names = []
    curr_label = 0
    for i, a in enumerate(datasets):
        labels += list(np.zeros(a.shape[0]) + curr_label)
        names.append(data_names[i])
        curr_label += 1
    labels = np.array(labels, dtype=int)

    embedding = visualize(datasets_dimred,
                          labels, NAMESPACE + '_ds', names,
                          perplexity=50, n_iter=400, size=10)


    # Uncorrected.
    datasets, genes = merge_datasets(datasets, genes_list)
    datasets = [ normalize(ds, axis=1) for ds in datasets ]
    datasets_dimred = dimensionality_reduce(datasets)
    
    embedding = visualize(datasets_dimred, labels,
                          NAMESPACE + '_ds_uncorrected', names,
                          perplexity=50, n_iter=400, size=10)



from fbpca import pca
from matplotlib import cm
import numpy as np
from scanorama import *
from scipy.sparse import vstack
import seaborn as sns
from time import time

from process import load_names, process

if __name__ == '__main__':
    from config import data_names

    datasets, genes_list, n_cells = load_names(data_names)
    datasets, genes = merge_datasets(datasets, genes_list)

    # Singular values.

    k = 300
    U, s, Vt = pca(vstack(datasets), k=k)
    
    xticklabels = [ str(i + 1) if (i+1) % 50 == 0 else ''
                    for i in range(k) ]
    xticklabels[-1] = str(k)
    
    plt.figure()
    sns.barplot(x=(np.array(range(len(s))) + 1), y=s, color='powderblue')
    plt.xticks(range(len(s)), xticklabels)
    plt.xlabel('Singular Value Rank')
    plt.ylabel('Singular Values')
    plt.savefig('top_sv.svg')
    
    # t-SNE of zero percentages.

    X = vstack(datasets)
    nonzero_transcripts = np.sum(X != 0, axis=1)
    zero_pct = nonzero_transcripts / float(X.shape[1])
    zero_pct = [ zero_pct[i, 0] for i in range(zero_pct.shape[0]) ]
    
    embedding = np.loadtxt('data/panorama_embedding.txt')
    
    plt.figure()
    plt.scatter(embedding[:, 0], embedding[:, 1],
                c=list(zero_pct), cmap=cm.get_cmap('Reds'),
                s=1, vmin=0, vmax=1)
    plt.xlabel('t-SNE 1')
    plt.ylabel('t-SNE 2')
    plt.savefig('zero_pct_tsne.svg')

    # Distribution of nonzero transcripts.
    
    process(data_names, min_trans=0)
    datasets, genes_list, n_cells = load_names(data_names)
    datasets, genes = merge_datasets(datasets, genes_list)
    
    X = vstack(datasets)
    nonzero_transcripts = np.sum(X != 0, axis=1)

    plt.figure()
    sns.distplot(nonzero_transcripts, kde=False, norm_hist=False)
    plt.xlabel('Number of transcripts')
    plt.ylabel('Cells')
    plt.savefig('transcript_distribution.svg')
    
    # Reset.
    process(data_names)




import numpy as np
from sklearn.mixture import GaussianMixture
from sklearn.preprocessing import normalize, LabelEncoder
import sys

from process import load_names
from scanorama import *

NAMESPACE = 'hsc'

data_names = [
    'data/hsc/hsc_mars',
    'data/hsc/hsc_ss2',
]

# Computes the probability that the corrected SS2 dataset
# comes from the original SS2 distribution or from the same
# distribution as the corrected MARS-Seq dataset.
if __name__ == '__main__':
    # Load data.
    datasets, genes_list, n_cells = load_names(data_names, verbose=False)
    datasets, genes = merge_datasets(datasets, genes_list, verbose=False)
    datasets, genes = process_data(datasets, genes)
    datasets = [ normalize(ds, axis=1) for ds in datasets ]
    
    # Fit initial mixture models.
    gm_ss2 = (GaussianMixture(n_components=3, n_init=3)
              .fit(datasets[1]))

    # Do batch correction.
    datasets = assemble(
        datasets,
        verbose=False, knn=KNN, sigma=SIGMA, approx=APPROX
    )
    datasets = [ normalize(ds, axis=1) for ds in datasets ]
    
    # Fit mixture models to other dataset.
    gm_mars_corrected = (
        GaussianMixture(n_components=3, n_init=3)
        .fit(datasets[0])
    )

    # Natural log likelihoods.
    ll_ss2 = gm_ss2.score(datasets[1])
    ll_mars_corrected = gm_mars_corrected.score(datasets[1])

    # Natural log of the likelihood ratio.
    print(ll_ss2 - max(ll_ss2, ll_mars_corrected))
    



import numpy as np
from scipy.sparse import vstack
from scipy.stats import ttest_ind
from sklearn.preprocessing import normalize, LabelEncoder
import sys

from process import load_names
from scanorama import correct, visualize, process_data, plt
from scanorama import dimensionality_reduce, merge_datasets

NAMESPACE = 'er_stress'

data_names = [
    'data/pancreas/pancreas_inDrop',
    'data/pancreas/pancreas_multi_celseq2_expression_matrix',
    'data/pancreas/pancreas_multi_celseq_expression_matrix',
    'data/pancreas/pancreas_multi_fluidigmc1_expression_matrix',
    'data/pancreas/pancreas_multi_smartseq2_expression_matrix',
]

if __name__ == '__main__':
    datasets, genes_list, n_cells = load_names(data_names)
    #datasets, genes = merge_datasets(datasets, genes_list)
    #datasets_dimred, genes = process_data(datasets, genes, hvg=hvg)
    datasets, genes = correct(datasets, genes_list)
    X = vstack(datasets).toarray()
    X[X < 0] = 0

    cell_labels = (
        open('data/cell_labels/pancreas_cluster.txt')
        .read().rstrip().split()
    )
    er_idx = [ i for i, cl in enumerate(cell_labels)
               if cl == 'beta_er' ]
    beta_idx = [ i for i, cl in enumerate(cell_labels)
                 if cl == 'beta' ]

    gadd_idx = list(genes).index('GADD45A')
    herp_idx = list(genes).index('HERPUD1')

    plt.figure()
    plt.boxplot([ X[er_idx, gadd_idx], X[beta_idx, gadd_idx] ],
                showmeans=True)
    plt.title('GADD45A (p = {})'.format(ttest_ind(
        X[er_idx, gadd_idx], X[beta_idx, gadd_idx], equal_var=False
    )[1]))
    plt.xticks([1, 2], ['beta_er', 'beta'])
    plt.ylabel('Scaled gene expression')
    plt.savefig('er_stress_GADD45A.svg')

    plt.figure()
    plt.boxplot([ X[er_idx, herp_idx], X[beta_idx, herp_idx] ],
                showmeans=True)
    plt.title('HERPUD1 (p = {})'.format(ttest_ind(
        X[er_idx, herp_idx], X[beta_idx, herp_idx], equal_var=False
    )[1]))
    plt.xticks([1, 2], ['beta_er', 'beta'])
    plt.ylabel('Scaled gene expression')
    plt.savefig('er_stress_HERPUD1.svg')



import numpy as np
from scanorama import *
from scipy.sparse import vstack
from scipy.stats import mannwhitneyu

from benchmark import write_table
from process import load_names, merge_datasets, process
from time_align import time_align_correlate, time_align_visualize, time_dist

NAMESPACE = 'mono_macro'

data_names = [
    'data/macrophage/monocytes',
    'data/pbmc/10x/cd14_monocytes',
    'data/macrophage/mcsf_day3_1',
    'data/macrophage/mcsf_day3_2',
    'data/macrophage/mcsf_day6_1',
    'data/macrophage/mcsf_day6_2',
]

def diff_expr(A, B, genes, permute_cutoff, verbose=True):

    p_vals = []
    for idx, gene in enumerate(genes):
        if sum(A[:, idx]) == 0 and sum(B[:, idx]) == 0:
            p_vals.append(1.)
            continue
        u, p = mannwhitneyu(A[:, idx], B[:, idx])
        p_vals.append(p)

    de_genes = []
    for idx, gene in enumerate(genes):
        if p_vals[idx] < permute_cutoff:
            if verbose:
                print('{}\t{}'.format(gene, p_vals[idx]))
            de_genes.append(gene)
    return de_genes

if __name__ == '__main__':
    from process import process
    process(data_names)
    
    datasets, genes_list, n_cells = load_names(data_names, norm=False)
    
    datasets, genes = merge_datasets(datasets, genes_list)
    datasets_dimred, genes = process_data(datasets, genes)
    
    _, A, _ = find_alignments_table(datasets_dimred)
    
    time = np.array([ 0, 0, 3, 3, 6, 6 ]).reshape(-1, 1)
    time_align_correlate(A, time)
    
    time_dist(datasets_dimred, time)

    x = np.array([ 0, 0, 1, 1, 2, 2 ]).reshape(-1, 1)
    y = [ -.1, .1, -.1, .1, -.1, .1 ]
    time_align_visualize(A, x, y, namespace=NAMESPACE)
    
    X = vstack(datasets).toarray()
    write_table(X, genes, 'data/macrophage/' + NAMESPACE, cell_name=NAMESPACE)
    
    de_genes = diff_expr(
        vstack(datasets[:2]).toarray(),
        vstack(datasets[4:]).toarray(),
        genes, 8.534789741091e-169, verbose=False
    )
    with open('data/macrophage/mono_macro_diffexpr_uncorrected.txt', 'w') as of:
        of.write('\n'.join(de_genes) + '\n')
    
    labels = []
    days = []
    curr_label = 0
    curr_day = 0
    for i, a in enumerate(datasets):
        labels += [ data_names[i] ] * int(a.shape[0])
        curr_label += 1
        if data_names[i] == 'data/macrophage/mcsf_day3_1':
            curr_day += 3
        if data_names[i] == 'data/macrophage/mcsf_day6_1':
            curr_day += 3            
        days += list(np.zeros(a.shape[0]) + curr_day)
    
    with open('data/macrophage/mono_macro_meta.txt', 'w') as of:
        of.write('Days\tBatch\n')
        for idx, day in enumerate(days):
            of.write('mono_macro{}\t{}\t{}'
                     .format(idx, int(day), labels[idx].split('/')[-1]) + '\n')
    
    with open('data/macrophage/' + NAMESPACE + '_genes.txt', 'w') as f:
        f.write('gene_short_name\n')
        for gene in genes:
            f.write('{0}\t{0}\n'.format(gene))
    
    assemble(datasets_dimred, expr_datasets=datasets)
    
    X = vstack(datasets).toarray()
    write_table(X, genes, 'data/macrophage/' + NAMESPACE + '_corrected',
                cell_name=NAMESPACE)
    
    de_genes = diff_expr(
        vstack(datasets[:2]).toarray(),
        vstack(datasets[4:]).toarray(),
        genes, 8.534789741091e-169, verbose=False
    )
    with open('data/macrophage/mono_macro_diffexpr_scanorama.txt', 'w') as of:
        of.write('\n'.join(de_genes) + '\n')



import numpy as np

from process import load_names, merge_datasets
from scanorama import process_data, find_alignments_table
from time_align import time_align_correlate, time_align_visualize, time_dist

NAMESPACE = 'fly_brain'

data_names = [
    'data/fly_brain/DGRP-551_0d_rep1',
    'data/fly_brain/DGRP-551_0d_rep2',
    'data/fly_brain/DGRP-551_1d_rep1',
    'data/fly_brain/DGRP-551_3d_rep1',
    'data/fly_brain/DGRP-551_6d_rep1',
    'data/fly_brain/DGRP-551_6d_rep2',
    'data/fly_brain/DGRP-551_9d_rep1',
    'data/fly_brain/DGRP-551_15d_rep1',
    'data/fly_brain/DGRP-551_30d_rep1',
    'data/fly_brain/DGRP-551_30d_rep2',
    'data/fly_brain/DGRP-551_50d_rep1',
]

if __name__ == '__main__':
    datasets, genes_list, n_cells = load_names(data_names)
    datasets, genes = merge_datasets(datasets, genes_list)
    datasets_dimred, genes = process_data(datasets, genes)

    _, A, _ = find_alignments_table(datasets_dimred)
    
    time = np.array([ 0, 0, 1, 3, 6, 6, 9, 15, 30, 30, 50 ]).reshape(-1, 1)
    time_align_correlate(A, time)
    
    time_dist(datasets_dimred, time)
    
    x = np.array([ 0, 0, 1, 2, 3, 3, 4, 5, 6, 6, 7 ]).reshape(-1, 1)
    y = [ -.1, .1, 0, 0, -.1, .1, 0, 0, -.1, .1, 0 ]
    time_align_visualize(A, x, y, namespace=NAMESPACE)




import numpy as np
from scanorama import *
from sklearn.manifold import TSNE
from sklearn.preprocessing import normalize
import sys
from time import time

from process import load_names

NAMESPACE = 'different'

data_names = [
    'data/293t_jurkat/293t',
    'data/brain/neuron_9k',
    'data/hsc/hsc_mars',
    'data/macrophage/uninfected',
    'data/pancreas/pancreas_inDrop',
    'data/pbmc/10x/68k_pbmc',
]

if __name__ == '__main__':
    datasets, genes_list, n_cells = load_names(data_names)

    datasets_dimred, datasets, genes = correct(
        datasets, genes_list, ds_names=data_names,
        return_dimred=True
    )

    labels = []
    names = []
    curr_label = 0
    for i, a in enumerate(datasets):
        labels += list(np.zeros(a.shape[0]) + curr_label)
        names.append(data_names[i])
        curr_label += 1
    labels = np.array(labels, dtype=int)
    
    visualize(datasets_dimred, labels, NAMESPACE, data_names)



import numpy as np
from scanorama import *
from scipy.sparse import vstack
from sklearn.preprocessing import normalize, LabelEncoder
import sys

from process import load_names

NAMESPACE = 'pbmc'

data_names = [
    'data/pbmc/10x/68k_pbmc',
    'data/pbmc/10x/b_cells',
    'data/pbmc/10x/cd14_monocytes',
    'data/pbmc/10x/cd4_t_helper',
    'data/pbmc/10x/cd56_nk',
    'data/pbmc/10x/cytotoxic_t',
    'data/pbmc/10x/memory_t',
    'data/pbmc/10x/regulatory_t',
    'data/pbmc/pbmc_kang',
    'data/pbmc/pbmc_10X',
]

if __name__ == '__main__':
    datasets, genes_list, n_cells = load_names(data_names)

    datasets_dimred, datasets, genes = correct(
        datasets, genes_list, ds_names=data_names,
        return_dimred=True
    )
    
    labels = []
    names = []
    curr_label = 0
    for i, a in enumerate(datasets):
        labels += list(np.zeros(a.shape[0]) + curr_label)
        names.append(data_names[i])
        curr_label += 1
    labels = np.array(labels, dtype=int)

    pbmc_genes = [
        'CD14', 'PTPRC', 'FCGR3A', 'ITGAX', 'ITGAM', 'CD19', 'HLA-DRB1',
        'FCGR2B', 'FCGR2A', 'CD3E', 'CD4', 'CD8A','CD8B', 'CD28', 'CD8',
        'TBX21', 'IKAROS', 'IL2RA', 'CD44', 'SELL', 'CCR7', 'MS4A1',
        'CD68', 'CD163', 'IL5RA', 'SIGLEC8', 'KLRD1', 'NCR1', 'CD22',
        'IL3RA', 'CCR6', 'IL7R', 'CD27', 'FOXP3', 'PTCRA', 'ID3', 'PF4',
        'CCR10', 'SIGLEC7', 'NKG7', 'S100A8', 'CXCR3', 'CCR5', 'CCR3',
        'CCR4', 'PTGDR2', 'RORC'
    ]
    
    embedding = visualize(datasets_dimred,
                          labels, NAMESPACE + '_ds', names,
                          #gene_names=pbmc_genes,
                          #gene_expr=vstack(datasets),
                          genes=genes, perplexity=500, n_iter=400)

    cell_labels = (
        open('data/cell_labels/pbmc_cluster.txt')
        .read().rstrip().split()
    )
    le = LabelEncoder().fit(cell_labels)
    cell_labels = le.transform(cell_labels)
    cell_types = le.classes_

    visualize(datasets_dimred,
              cell_labels, NAMESPACE + '_type', cell_types,
              embedding=embedding)
    
    # Uncorrected.
    datasets, genes_list, n_cells = load_names(data_names)
    datasets, genes = merge_datasets(datasets, genes_list)
    datasets = [ normalize(ds, axis=1) for ds in datasets ]
    datasets_dimred = dimensionality_reduce(datasets)
    
    embedding = visualize(datasets_dimred, labels,
                          NAMESPACE + '_ds_uncorrected', names,
                          perplexity=100, n_iter=400)
    visualize(None, cell_labels,
              NAMESPACE + '_type_uncorrected', cell_types,
              embedding=embedding,
              perplexity=100, n_iter=400)



import gzip
import numpy as np
import os.path
import scipy.sparse
from scipy.sparse import csr_matrix
from sklearn.preprocessing import normalize
import sys

from scanorama import merge_datasets

MIN_TRANSCRIPTS = 600

def load_tab(fname, max_genes=40000):
    if fname.endswith('.gz'):
        opener = gzip.open
    else:
        opener = open
        
    with opener(fname, 'r') as f:
        if fname.endswith('.gz'):
            header = f.readline().decode('utf-8').rstrip().split('\t')
        else:
            header = f.readline().rstrip().split('\t')
            
        cells = header[1:]
        X = np.zeros((len(cells), max_genes))
        genes = []
        for i, line in enumerate(f):
            if i > max_genes:
                break
            if fname.endswith('.gz'):
                line = line.decode('utf-8')
            fields = line.rstrip().split('\t')
            genes.append(fields[0])
            X[:, i] = [ float(f) for f in fields[1:] ]
    return X[:, range(len(genes))], np.array(cells), np.array(genes)

def load_mtx(dname):
    with open(dname + '/matrix.mtx', 'r') as f:
        while True:
            header = f.readline()
            if not header.startswith('%'):
                break
        header = header.rstrip().split()
        n_genes, n_cells = int(header[0]), int(header[1])

        data, i, j = [], [], []
        for line in f:
            fields = line.rstrip().split()
            data.append(float(fields[2]))
            i.append(int(fields[1])-1)
            j.append(int(fields[0])-1)
        X = csr_matrix((data, (i, j)), shape=(n_cells, n_genes))

    genes = []
    with open(dname + '/genes.tsv', 'r') as f:
        for line in f:
            fields = line.rstrip().split()
            genes.append(fields[1])
    assert(len(genes) == n_genes)

    return X, np.array(genes)

def load_h5(fname, genome='mm10'):
    try:
        import tables
    except ImportError:
        sys.stderr.write('Please install PyTables to read .h5 files: '
                         'https://www.pytables.org/usersguide/installation.html\n')
        exit(1)
    
    # Adapted from scanpy's read_10x_h5() method.
    with tables.open_file(str(fname), 'r') as f:
        try:
            dsets = {}
            for node in f.walk_nodes('/' + genome, 'Array'):
                dsets[node.name] = node.read()

            n_genes, n_cells = dsets['shape']
            data = dsets['data']
            if dsets['data'].dtype == np.dtype('int32'):
                data = dsets['data'].view('float32')
                data[:] = dsets['data']

            X = csr_matrix((data, dsets['indices'], dsets['indptr']),
                           shape=(n_cells, n_genes))
            genes = [ gene for gene in dsets['genes'].astype(str) ]
            assert(len(genes) == n_genes)
            assert(len(genes) == X.shape[1])
            
        except tables.NoSuchNodeError:
            raise Exception('Genome %s does not exist in this file.' % genome)
        except KeyError:
            raise Exception('File is missing one or more required datasets.')    

    return X, np.array(genes)
        
def process_tab(fname, min_trans=MIN_TRANSCRIPTS):
    X, cells, genes = load_tab(fname)

    gt_idx = [ i for i, s in enumerate(np.sum(X != 0, axis=1))
               if s >= min_trans ]
    X = X[gt_idx, :]
    cells = cells[gt_idx]
    if len(gt_idx) == 0:
        print('Warning: 0 cells passed QC in {}'.format(fname))
    if fname.endswith('.txt'):
        cache_prefix = '.'.join(fname.split('.')[:-1])
    elif fname.endswith('.txt.gz'):
        cache_prefix = '.'.join(fname.split('.')[:-2])
    elif fname.endswith('.tsv'):
        cache_prefix = '.'.join(fname.split('.')[:-1])
    elif fname.endswith('.tsv.gz'):
        cache_prefix = '.'.join(fname.split('.')[:-2])
    else:
        sys.stderr.write('Tab files should end with ".txt" or ".tsv"\n')
        exit(1)
        
    cache_fname = cache_prefix + '.npz'
    np.savez(cache_fname, X=X, genes=genes)

    return X, cells, genes

def process_mtx(dname, min_trans=MIN_TRANSCRIPTS):
    X, genes = load_mtx(dname)

    gt_idx = [ i for i, s in enumerate(np.sum(X != 0, axis=1))
               if s >= min_trans ]
    X = X[gt_idx, :]
    if len(gt_idx) == 0:
        print('Warning: 0 cells passed QC in {}'.format(dname))
    
    cache_fname = dname + '/tab.npz'
    scipy.sparse.save_npz(cache_fname, X, compressed=False)

    with open(dname + '/tab.genes.txt', 'w') as of:
        of.write('\n'.join(genes) + '\n')

    return X, genes

def process_h5(fname, min_trans=MIN_TRANSCRIPTS):
    X, genes = load_h5(fname)

    gt_idx = [ i for i, s in enumerate(np.sum(X != 0, axis=1))
               if s >= min_trans ]
    X = X[gt_idx, :]
    if len(gt_idx) == 0:
        print('Warning: 0 cells passed QC in {}'.format(fname))
    
    if fname.endswith('.h5'):
        cache_prefix = '.'.join(fname.split('.')[:-1])
        
    cache_fname = cache_prefix + '.h5.npz'
    scipy.sparse.save_npz(cache_fname, X, compressed=False)

    with open(cache_prefix + '.h5.genes.txt', 'w') as of:
        of.write('\n'.join(genes) + '\n')

    return X, genes

def load_data(name):
    if os.path.isfile(name + '.h5.npz'):
        X = scipy.sparse.load_npz(name + '.h5.npz')
        with open(name + '.h5.genes.txt') as f:
            genes = np.array(f.read().rstrip().split())
    elif os.path.isfile(name + '.npz'):
        data = np.load(name + '.npz')
        X = data['X']
        genes = data['genes']
        data.close()
    elif os.path.isfile(name + '/tab.npz'):
        X = scipy.sparse.load_npz(name + '/tab.npz')
        with open(name + '/tab.genes.txt') as f:
            genes = np.array(f.read().rstrip().split())
    else:
        sys.stderr.write('Could not find: {}\n'.format(name))
        exit(1)
    genes = np.array([ gene.upper() for gene in genes ])
    return X, genes

def load_names(data_names, norm=True, log1p=False, verbose=True):
    # Load datasets.
    datasets = []
    genes_list = []
    n_cells = 0
    for name in data_names:
        X_i, genes_i = load_data(name)
        if norm:
            X_i = normalize(X_i, axis=1)
        if log1p:
            X_i = np.log1p(X_i)
        X_i = csr_matrix(X_i)
            
        datasets.append(X_i)
        genes_list.append(genes_i)
        n_cells += X_i.shape[0]
        if verbose:
            print('Loaded {} with {} genes and {} cells'.
                  format(name, X_i.shape[1], X_i.shape[0]))
    if verbose:
        print('Found {} cells among all datasets'
              .format(n_cells))

    return datasets, genes_list, n_cells

def save_datasets(datasets, genes, data_names, verbose=True,
                  truncate_neg=False):
    for i in range(len(datasets)):
        dataset = datasets[i].toarray()
        name = data_names[i]

        if truncate_neg:
            dataset[dataset < 0] = 0

        with open(name + '.scanorama_corrected.txt', 'w') as of:
            # Save header.
            of.write('Genes\t')
            of.write('\t'.join(
                [ 'cell' + str(cell) for cell in range(dataset.shape[0]) ]
            ) + '\n')

            for g in range(dataset.shape[1]):
                of.write(genes[g] + '\t')
                of.write('\t'.join(
                    [ str(expr) for expr in dataset[:, g] ]
                ) + '\n')

def process(data_names, min_trans=MIN_TRANSCRIPTS):
    for name in data_names:
        if os.path.isdir(name):
            process_mtx(name, min_trans=min_trans)
        elif os.path.isfile(name) and name.endswith('.h5'):
            process_h5(name, min_trans=min_trans)
        elif os.path.isfile(name + '.h5'):
            process_h5(name + '.h5', min_trans=min_trans)
        elif os.path.isfile(name):
            process_tab(name, min_trans=min_trans)
        elif os.path.isfile(name + '.txt'):
            process_tab(name + '.txt', min_trans=min_trans)
        elif os.path.isfile(name + '.txt.gz'):
            process_tab(name + '.txt.gz', min_trans=min_trans)
        elif os.path.isfile(name + '.tsv'):
            process_tab(name + '.tsv', min_trans=min_trans)
        elif os.path.isfile(name + '.tsv.gz'):
            process_tab(name + '.tsv.gz', min_trans=min_trans)
        else:
            sys.stderr.write('Warning: Could not find {}\n'.format(name))
            continue
        print('Successfully processed {}'.format(name))

if __name__ == '__main__':
    from config import data_names

    process(data_names)



import scanorama
import numpy as np

def data_gen():
    X1 = np.random.rand(100, 10)
    genes1 = [ 'g' + str(i) for i in range(10) ]

    X2 = np.random.rand(200, 12)
    genes2 = list(reversed([ 'g' + str(i) for i in range(12) ]))

    return [ X1, X2 ], [ genes1, genes2 ]

def test_scanorama_integrate():
    """
    Test that Scanorama integration works.
    Ensures that the function call runs and that the returned
    dimensions match up.
    """

    datasets, genes_list = data_gen()

    integrated, genes = scanorama.integrate(datasets, genes_list)

    for X_int, X_orig in zip(integrated, datasets):
        assert(X_int.shape[0] == X_orig.shape[0])
        assert(X_int.shape[1] == len(genes))

def test_scanorama_integrate_scanpy():
    """
    Test that Scanorama integration with Scanpy AnnData works.
    Ensures that the function call runs and dimensions match.
    """
    from anndata import AnnData
    import pandas as pd

    datasets, genes_list = data_gen()

    adatas = []
    for i in range(len(datasets)):
        adata = AnnData(datasets[i])
        adata.obs = pd.DataFrame(list(range(datasets[i].shape[0])),
                                 columns=[ 'obs1' ])
        adata.var = pd.DataFrame(genes_list[i], columns=[ 'var1' ])
        adatas.append(adata)

    scanorama.integrate_scanpy(adatas)

    for adata, X in zip(adatas, datasets):
        assert(adata.obsm['X_scanorama'].shape[0] == X.shape[0])

def test_scanorama_correct_scanpy():
    """
    Test that Scanorama correction with Scanpy AnnData works.
    Ensures that the function call runs, dimensions match, and
    metadata is in the correct order.
    """
    from anndata import AnnData
    import pandas as pd

    datasets, genes_list = data_gen()

    adatas = []
    for i in range(len(datasets)):
        adata = AnnData(datasets[i])
        adata.obs = pd.DataFrame(list(range(datasets[i].shape[0])),
                                 columns=[ 'obs1' ])
        adata.var = pd.DataFrame(genes_list[i], columns=[ 'var1' ])
        adata.var_names = genes_list[i]
        adatas.append(adata)

    corrected = scanorama.correct_scanpy(adatas)

    for adata_cor, adata_orig in zip(corrected, adatas):
        assert(adata_cor.X.shape[0] == adata_orig.X.shape[0])
        assert(adata_cor.X.shape[1] == adatas[0].X.shape[1])
        assert(list(adata_cor.obs['obs1']) == list(adata_orig.obs['obs1']))
        assert(list(adata_cor.var['var1']) == list(adatas[0].var['var1']))



from process import load_names
from scanorama import *

from time import time
import numpy as np

NAMESPACE = 'panorama_integrated'

if __name__ == '__main__':
    from config import data_names

    datasets, genes_list, n_cells = load_names(data_names)
    datasets, genes = merge_datasets(datasets, genes_list)

    labels = []
    names = []
    curr_label = 0
    for i, a in enumerate(datasets):
        labels += list(np.zeros(a.shape[0]) + curr_label)
        names.append(data_names[i])
        curr_label += 1
    labels = np.array(labels, dtype=int)
    
    ########################
    ## Scanorama Assembly ##
    ########################

    datasets_dimred, genes = process_data(datasets, genes)
    
    # Put each of the datasets into a panorama.
    t0 = time()
    datasets_dimred = assemble(
        datasets_dimred, ds_names=data_names, sigma=150,
    )
    if VERBOSE:
        print('Integrated panoramas in {:.3f}s'.format(time() - t0))

    embedding = visualize(
        datasets_dimred, labels, NAMESPACE, data_names,
        multicore_tsne=False
    )

    np.savetxt('data/{}_embedding.txt'.format(NAMESPACE),
               embedding, delimiter='\t')



import numpy as np
from scanorama import *
from scipy.stats import ttest_ind
from sklearn.metrics import silhouette_samples as sil

from process import load_names, process

def test_knn(datasets_dimred, genes, labels, idx, distr, xlabels):
    knns = [ 5, 10, 50, 100 ]
    len_distr = len(distr)
    for knn in knns:
        integrated = assemble(datasets_dimred[:], knn=knn, sigma=150)
        X = np.concatenate(integrated)
        distr.append(sil(X[idx, :], labels[idx]))
        for d in distr[:len_distr]:
            print(ttest_ind(np.ravel(X[idx, :]), np.ravel(d)))
        xlabels.append(str(knn))
    print('')
    
    plt.figure()
    plt.boxplot(distr, showmeans=True, whis='range')
    plt.xticks(range(1, len(xlabels) + 1), xlabels)
    plt.ylabel('Silhouette Coefficient')
    plt.ylim((-1, 1))
    plt.savefig('param_sensitivity_{}.svg'.format('knn'))

def test_sigma(datasets_dimred, genes, labels, idx, distr, xlabels):
    sigmas = [ 10, 50, 100, 200 ]
    len_distr = len(distr)
    for sigma in sigmas:
        integrated = assemble(datasets_dimred[:], sigma=sigma)
        X = np.concatenate(integrated)
        distr.append(sil(X[idx, :], labels[idx]))
        for d in distr[:len_distr]:
            print(ttest_ind(np.ravel(X[idx, :]), np.ravel(d)))
        xlabels.append(str(sigma))
    print('')
    
    plt.figure()
    plt.boxplot(distr, showmeans=True, whis='range')
    plt.xticks(range(1, len(xlabels) + 1), xlabels)
    plt.ylabel('Silhouette Coefficient')
    plt.ylim((-1, 1))
    plt.savefig('param_sensitivity_{}.svg'.format('sigma'))

def test_alpha(datasets_dimred, genes, labels, idx, distr, xlabels):
    alphas = [ 0, 0.05, 0.20, 0.50 ]
    len_distr = len(distr)
    for alpha in alphas:
        integrated = assemble(datasets_dimred[:], alpha=alpha, sigma=150)
        X = np.concatenate(integrated)
        distr.append(sil(X[idx, :], labels[idx]))
        for d in distr[:len_distr]:
            print(ttest_ind(np.ravel(X[idx, :]), np.ravel(d)))
        xlabels.append(str(alpha))
    print('')
    
    plt.figure()
    plt.boxplot(distr, showmeans=True, whis='range')
    plt.xticks(range(1, len(xlabels) + 1), xlabels)
    plt.ylabel('Silhouette Coefficient')
    plt.ylim((-1, 1))
    plt.savefig('param_sensitivity_{}.svg'.format('alpha'))

def test_approx(datasets_dimred, genes, labels, idx, distr, xlabels):
    integrated = assemble(datasets_dimred[:], approx=False, sigma=150)
    X = np.concatenate(integrated)
    distr.append(sil(X[idx, :], labels[idx]))
    len_distr = len(distr)
    for d in distr[:len_distr]:
        print(ttest_ind(np.ravel(X[idx, :]), np.ravel(d)))
    xlabels.append('Exact NN')
    print('')
    
    plt.figure()
    plt.boxplot(distr, showmeans=True, whis='range')
    plt.xticks(range(1, len(xlabels) + 1), xlabels)
    plt.ylabel('Silhouette Coefficient')
    plt.ylim((-1, 1))
    plt.savefig('param_sensitivity_{}.svg'.format('approx'))

def fit_tsne(X, perplexity=PERPLEXITY, n_iter=N_ITER,
             learn_rate=200., early_exag=12.):
    try:
        from MulticoreTSNE import MulticoreTSNE
        tsne = MulticoreTSNE(
            n_iter=500, perplexity=perplexity,
            learning_rate=learn_rate,
            early_exaggeration=early_exag,
            random_state=69,
            n_jobs=40
        )
    except ImportError:
        tsne = TSNEApprox(
            n_iter=500, perplexity=perplexity,
            learning_rate=learn_rate,
            early_exaggeration=early_exag,
            random_state=69,
        )
    tsne.fit(X)
    embedding = tsne.embedding_
    return embedding

def test_perplexity(datasets_dimred, genes, labels, idx,
                    distr, xlabels):
    X = np.concatenate(datasets_dimred)

    perplexities = [ 10, 100, 500, 2000 ]
    len_distr = len(distr)
    for perplexity in perplexities:
        embedding = fit_tsne(X, perplexity=perplexity)
        distr.append(sil(embedding[idx, :], labels[idx]))
        for d in distr[:len_distr]:
            print(ttest_ind(np.ravel(X[idx, :]), np.ravel(d)))
        xlabels.append(str(perplexity))
    print('')
    
    plt.figure()
    plt.boxplot(distr, showmeans=True, whis='range')
    plt.xticks(range(1, len(xlabels) + 1), xlabels)
    plt.ylabel('Silhouette Coefficient')
    plt.ylim((-1, 1))
    plt.savefig('param_sensitivity_{}.svg'.format('perplexity'))

def test_learn_rate(datasets_dimred, genes, labels, idx,
                    distr, xlabels):
    X = np.concatenate(datasets_dimred)

    learn_rates = [ 50., 100., 500., 1000. ]
    len_distr = len(distr)
    for learn_rate in learn_rates:
        embedding = fit_tsne(X, learn_rate=learn_rate)
        distr.append(sil(embedding[idx, :], labels[idx]))
        for d in distr[:len_distr]:
            print(ttest_ind(np.ravel(X[idx, :]), np.ravel(d)))
        xlabels.append(str(learn_rate))
    print('')
    
    plt.figure()
    plt.boxplot(distr, showmeans=True, whis='range')
    plt.xticks(range(1, len(xlabels) + 1), xlabels)
    plt.ylabel('Silhouette Coefficient')
    plt.ylim((-1, 1))
    plt.savefig('param_sensitivity_{}.svg'.format('learn_rate'))

if __name__ == '__main__':
    with open('conf/panorama.txt') as f:
        data_names = f.read().split()

    labels = np.array(
        open('data/cell_labels/all.txt').read().rstrip().split()
    )
    idx = range(labels.shape[0])
    
    datasets, genes_list, n_cells = load_names(data_names)
    datasets, genes = merge_datasets(datasets, genes_list)
    datasets_dimred, genes = process_data(datasets, genes)

    X = np.concatenate(datasets_dimred)
    sil_non = sil(X[idx, :], labels[idx])
    print(np.median(sil_non))
    
    X = np.loadtxt('data/corrected_mnn.txt')
    sil_mnn = sil(X[idx, :], labels[idx])
    print(np.median(sil_mnn))

    X = np.loadtxt('data/corrected_seurat.txt')
    sil_cca = sil(X[idx, :], labels[idx])
    print(np.median(sil_cca))

    distr = [  sil_non, sil_mnn, sil_cca ]
    xlabels = [ 'No correction', 'scran MNN', 'Seurat CCA' ]

    # Test alignment parameters.
    test_approx(datasets_dimred[:], genes, labels, idx, distr[:], xlabels[:])
    test_alpha(datasets_dimred[:], genes, labels, idx, distr[:], xlabels[:])
    test_knn(datasets_dimred[:], genes, labels, idx, distr[:], xlabels[:])
    test_sigma(datasets_dimred[:], genes, labels, idx, distr[:], xlabels[:])

    datasets_dimred = assemble(datasets_dimred)
    
    # Test visualization parameters.
    test_perplexity(datasets_dimred[:], genes, labels, idx, distr[:], xlabels[:])
    test_learn_rate(datasets_dimred[:], genes, labels, idx, distr[:], xlabels[:])



from config import data_names

from scanorama import *
from sklearn.metrics import silhouette_samples as sil
from process import load_names, merge_datasets

from time import time
import numpy as np

NAMESPACE = 'panorama1'

# Find the panoramas in the data.
def panorama(datasets_full, genes_list):
    if VERBOSE:
        print('Processing and reducing dimensionality...')
    datasets, genes = merge_datasets(datasets_full, genes_list)
    datasets_dimred, genes = process_data(datasets, genes)

    if VERBOSE:
        print('Finding panoramas...')
    panoramas = connect(datasets_dimred)

    if VERBOSE:
        print(panoramas)

    return panoramas

if __name__ == '__main__':
    # Load raw data from files.
    datasets, genes_list, n_cells = load_names(data_names)

    ##########################
    ## Panorama correction. ##
    ##########################

    # Put each of the datasets into a panorama.
    t0 = time()
    panoramas = panorama(datasets[:], genes_list)
    if VERBOSE:
        print('Found panoramas in {:.3f}s'.format(time() - t0))

    # Assemble and correct each panorama individually.
    t0 = time()
    for p_idx, pano in enumerate(panoramas):
        if VERBOSE:
            print('Building panorama {}'.format(p_idx))

        # Consider just the datasets in the panorama.
        pan_datasets = [ datasets[p] for p in pano ]
        pan_genes_list = [ genes_list[p] for p in pano ]

        # Do batch correction on those datasets.
        pan_datasets, genes = correct(pan_datasets, pan_genes_list, sigma=150)
        if VERBOSE:
            print('Found {} genes in panorama'.format(len(genes)))

        # Store batch corrected result.
        for i, p in enumerate(pano):
            datasets[p] = pan_datasets[i]
            genes_list[p] = genes

    if VERBOSE:
        print('Batch corrected panoramas in {:.3f}s'.format(time() - t0))

    ##########################
    ## Final visualization. ##
    ##########################

    # Put all the corrected data together for visualization purposes.
    datasets, genes = merge_datasets(datasets, genes_list)
    datasets_dimred, genes = process_data(datasets, genes)

    # Label each cell with its dataset of origin.
    labels = np.zeros(n_cells, dtype=int)
    base = 0
    for i, ds in enumerate(datasets):
        labels[base:(base + ds.shape[0])] = i
        base += ds.shape[0]

    embedding = visualize(
        datasets_dimred, labels, NAMESPACE, data_names,
        n_iter=450, multicore_tsne=False
    )
    np.savetxt('data/{}_embedding.txt'.format(NAMESPACE),
               embedding, delimiter='\t')



import numpy as np
from scanorama import plt
plt.rcParams.update({'font.size': 25})
import sys

scano, mnn, uncor = {}, {}, {}

in_scano = True
in_mnn = False
for line in open(sys.argv[1]):
    fields = line.rstrip().split()
    if len(fields) > 3 or len(fields) < 2:
        continue
    try:
        F = float(fields[1])
    except ValueError:
        continue

    gene = fields[0]

    if gene.startswith("b'"):
        gene = gene[2:].replace("'", '')

    if in_scano:
        scano[gene] = F
    elif in_mnn:
        mnn[gene] = F
    else:
        uncor[gene] = F

    if gene == 'ZZZ3':
        if in_scano:
            in_scano = False
            in_mnn = True
        elif in_mnn:
            in_mnn = False

# Scanorama.

scanorama, uncorrected = [], []
for gene in set(scano.keys()) & set(uncor.keys()):
    scanorama.append(scano[gene])
    uncorrected.append(uncor[gene])
scanorama = np.array(scanorama)
uncorrected = np.array(uncorrected)

below = sum(scanorama > uncorrected + 50)
above = sum(scanorama < uncorrected - 50)

print('{}% above line'.format(float(above) / float(above + below) * 100))
    
name = sys.argv[1].split('.')[0]
line = max(min(max(scanorama), max(uncorrected)), 2100)

from scipy.stats import pearsonr
print(pearsonr(scanorama, uncorrected))
        
plt.figure()
plt.scatter(scanorama, uncorrected, s=10)
plt.plot([0, line], [0, line], 'r--')
plt.xlim([ 0, 2100 ])
plt.tight_layout()
plt.savefig('oneway_scanorama.png')

# scran MNN.

scranmnn, uncorrected = [], []
for gene in set(mnn.keys()) & set(uncor.keys()):
    scranmnn.append(mnn[gene])
    uncorrected.append(uncor[gene])
scranmnn = np.array(scranmnn)
uncorrected = np.array(uncorrected)

below = sum(scranmnn > uncorrected + 50)
above = sum(scranmnn < uncorrected - 50)

print('{}% above line'.format(float(above) / float(above + below) * 100))
    
name = sys.argv[1].split('.')[0]
line = max(min(max(scranmnn), max(uncorrected)), 2100)

from scipy.stats import pearsonr
print(pearsonr(scranmnn, uncorrected))
        
plt.figure()
plt.scatter(scranmnn, uncorrected, s=10)
plt.plot([0, line], [0, line], 'r--')
plt.xlim([ 0, 2100 ])
plt.tight_layout()
plt.savefig('oneway_mnn.png')




import networkx as nx
import numpy as np
from scipy.stats import pearsonr, spearmanr
import seaborn as sns
from sklearn.metrics.pairwise import euclidean_distances
import sys

from scanorama import plt

def time_align_visualize(alignments, time, y, namespace='time_align'):
    plt.figure()
    heat = np.flip(alignments + alignments.T +
                   np.eye(alignments.shape[0]), axis=0)
    sns.heatmap(heat, cmap="YlGnBu", vmin=0, vmax=1)
    plt.savefig(namespace + '_heatmap.svg')

    G = nx.from_numpy_matrix(alignments)
    G = nx.maximum_spanning_tree(G)

    pos = {}
    for i in range(len(G.nodes)):
        pos[i] = np.array([time[i], y[i]])

    mst_edges = set(nx.maximum_spanning_tree(G).edges())
    
    weights = [ G[u][v]['weight'] if (not (u, v) in mst_edges) else 8
                for u, v in G.edges() ]
    
    plt.figure()
    nx.draw(G, pos, edges=G.edges(), width=10)
    plt.ylim([-1, 1])
    plt.savefig(namespace + '.svg')

def time_align_correlate(alignments, time):
    time_dist = euclidean_distances(time, time)

    assert(time_dist.shape == alignments.shape)

    time_dists, scores = [], []
    for i in range(time_dist.shape[0]):
        for j in range(time_dist.shape[1]):
            if i >= j:
                continue
            time_dists.append(time_dist[i, j])
            scores.append(alignments[i, j])

    print('Spearman rho = {}'.format(spearmanr(time_dists, scores)))
    print('Pearson rho = {}'.format(pearsonr(time_dists, scores)))
    
def load_alignments_from_log(fname):
    names = []
    alignments = []
    with open(fname) as f:
        row = []
        in_row = False
        while True:
            line = f.readline().strip()
            if not line:
                break
            
            if line.startswith('Loaded'):
                names.append(line.split()[1])
                continue
                
            if line.startswith('[') and not 't-SNE' in line:
                fields = line.replace('[', '').replace(']', '').strip()
                row += [ float(field) for field in fields.split() ]
                in_row = True
                if line.endswith(']'):
                    alignments.append(row)
                    row = []
                    in_row = False
                    continue

            if line.endswith(']'):
                fields = line.replace('[', '').replace(']', '').strip()
                row += [ float(field) for field in fields.split() ]
                if len(alignments) > 0:
                    assert(len(alignments[-1]) == len(row))
                alignments.append(row)
                row = []
                in_row = False

            if in_row and not line.startswith('['):
                fields = line.replace('[', '').replace(']', '').strip()
                row += [ float(field) for field in fields.split() ]

    A = np.array(alignments)
    
    return A, names

def time_dist(datasets_dimred, time):
    time_dist = euclidean_distances(time, time)

    time_dists, scores = [], []
    for i in range(time_dist.shape[0]):
        for j in range(time_dist.shape[1]):
            if i >= j:
                continue
            score = np.mean(euclidean_distances(
                datasets_dimred[i], datasets_dimred[j]
            ))
            time_dists.append(time_dist[i, j])
            scores.append(score)

    print('Spearman rho = {}'.format(spearmanr(time_dists, scores)))
    print('Pearson rho = {}'.format(pearsonr(time_dists, scores)))

if __name__ == '__main__':
    pass



import numpy as np
from scanorama import *
from scipy.sparse import vstack
from sklearn.preprocessing import normalize, LabelEncoder
import sys

from process import load_names

NAMESPACE = 'hsc'

data_names = [
    'data/hsc/hsc_mars',
    'data/hsc/hsc_ss2',
]

if __name__ == '__main__':
    datasets, genes_list, n_cells = load_names(data_names)

    datasets, genes = correct(datasets, genes_list)
    datasets_dimred = dimensionality_reduce(datasets, dimred=2)

    labels = []
    names = []
    curr_label = 0
    for i, a in enumerate(datasets):
        labels += list(np.zeros(a.shape[0]) + curr_label)
        names.append(data_names[i])
        curr_label += 1
    labels = np.array(labels, dtype=int)

    hsc_genes = [
        'GATA2', 'APOE', 'SPHK1', 'CTSE', 'FOS'
    ]

    # Visualize with PCA.
    visualize(None, labels, NAMESPACE + '_ds', names,
              gene_names=hsc_genes, genes=genes,
              gene_expr=vstack(datasets),
              embedding=np.concatenate(datasets_dimred),
              size=4)

    cell_labels = (
        open('data/cell_labels/hsc_cluster.txt')
        .read().rstrip().split()
    )
    le = LabelEncoder().fit(cell_labels)
    cell_labels = le.transform(cell_labels)
    cell_types = le.classes_

    visualize(None, cell_labels, NAMESPACE + '_type', cell_types,
              embedding=np.concatenate(datasets_dimred), size=4)

    # Uncorrected.
    datasets, genes_list, n_cells = load_names(data_names)
    datasets, genes = merge_datasets(datasets, genes_list)
    datasets = [ normalize(ds, axis=1) for ds in datasets ]
    datasets_dimred = dimensionality_reduce(datasets, dimred=2)
    
    visualize(None, labels, NAMESPACE + '_ds_uncorrected', names,
              embedding=np.concatenate(datasets_dimred), size=4)
    visualize(None, cell_labels, NAMESPACE + '_type_uncorrected', cell_types,
              embedding=np.concatenate(datasets_dimred), size=4)



from setuptools import setup, find_packages

setup(
    name='scanorama',
    version='1.7.1',
    description='Panoramic stitching of heterogeneous single cell transcriptomic data',
    url='https://github.com/brianhie/scanorama',
    download_url='https://github.com/brianhie/scanorama/archive/v1.7.1.tar.gz',
    packages=find_packages(exclude=['bin', 'conf', 'data', 'target']),
    install_requires=[
        'annoy>=1.11.5',
        'fbpca>=1.0',
        'geosketch>=1.0',
        'intervaltree==2.1.0',
        'matplotlib>=2.0.2',
        'numpy>=1.12.0',
        'scipy>=1.0.0',
        'scikit-learn>=0.20rc1',
    ],
    author='Brian Hie',
    author_email='brianhie@mit.edu',
    license='MIT'
)



import numpy as np
from scanorama import *
from scipy.stats import ttest_ind
from sklearn.metrics import silhouette_samples as sil

from process import load_names

if __name__ == '__main__':
    with open('conf/panorama.txt') as f:
        data_names = f.read().rstrip().split()

    labels = np.array(
        open('data/cell_labels/all.txt').read().rstrip().split()
    )
    idx = range(labels.shape[0])
    #idx = np.random.choice(len(labels), size=int(len(labels)/2), replace=False)
    
    datasets, genes_list, n_cells = load_names(data_names)
    datasets, genes = merge_datasets(datasets, genes_list)
    datasets_dimred, genes = process_data(datasets, genes)

    # Baseline without correction.
    X = np.concatenate(datasets_dimred)
    sil_non = sil(X[idx, :], labels[idx])
    print(np.median(sil_non))

    # scran MNN.
    X = np.loadtxt('data/corrected_mnn.txt')
    sil_mnn = sil(X[idx, :], labels[idx])
    print(np.median(sil_mnn))

    # Seurat CCA.
    X = np.loadtxt('data/corrected_seurat.txt')
    sil_cca = sil(X[idx, :], labels[idx])
    print(np.median(sil_cca))

    # Scanorama.
    X = np.concatenate(assemble(datasets_dimred, sigma=150))
    sil_pan = sil(X[idx, :], labels[idx])
    print(np.median(sil_pan))

    print(ttest_ind(sil_pan, sil_non))
    print(ttest_ind(sil_pan, sil_cca))
    print(ttest_ind(sil_pan, sil_mnn))
    
    plt.figure()
    plt.boxplot([ sil_non, sil_mnn, sil_cca, sil_pan ], showmeans=True, whis='range')
    plt.ylim([ -1, 1 ])
    plt.title('Distributions of Silhouette Coefficients')
    plt.xticks(range(1, 5), [ 'No correction', 'scran MNN', 'Seurat CCA', 'Scanorama' ])
    plt.ylabel('Silhouette Coefficient')
    plt.savefig('silhouette.svg')



import numpy as np
from scanorama import *
from scipy.sparse import vstack
from sklearn.preprocessing import normalize, LabelEncoder
import sys
from time import time

from benchmark import write_table
from process import load_names, process

np.random.seed(0)

NAMESPACE = 'mouse_brain_sketched'
BATCH_SIZE = 10000

data_names = [
    'data/mouse_brain/nuclei',
    'data/mouse_brain/dropviz/Cerebellum_ALT',
    'data/mouse_brain/dropviz/Cortex_noRep5_FRONTALonly',
    'data/mouse_brain/dropviz/Cortex_noRep5_POSTERIORonly',
    'data/mouse_brain/dropviz/EntoPeduncular',
    'data/mouse_brain/dropviz/GlobusPallidus',
    'data/mouse_brain/dropviz/Hippocampus',
    'data/mouse_brain/dropviz/Striatum',
    'data/mouse_brain/dropviz/SubstantiaNigra',
    'data/mouse_brain/dropviz/Thalamus',
]

if __name__ == '__main__':
    process(data_names, min_trans=100)

    datasets, genes_list, n_cells = load_names(data_names)

    datasets_merged, genes = merge_datasets(datasets[:], genes_list)

    t0 = time()
    datasets_dimred, genes = integrate(
        datasets, genes_list, ds_names=data_names,
        sketch=True, sketch_method='geosketch', sketch_max=2000,
    )
    print('Sketched and integrated panoramas in {:.3f}s'
          .format(time() - t0))

    datasets = datasets_merged

    names = []
    for i, a in enumerate(datasets_dimred):
        names.append(data_names[i])

    mouse_brain_genes = [
        'Gja1', 'Flt1', 'Gabra6', 'Syt1', 'Gabrb2', 'Gabra1',
        'Meg3', 'Mbp', 'Rgs5',
    ]

    # Downsample for visualization purposes
    rand_idxs = []
    labels = []
    curr_label = 0
    for i in range(len(data_names)):
        ds = datasets_dimred[i]
        rand_idx = np.random.choice(ds.shape[0], size=int(ds.shape[0]/10),
                                    replace=False)
        datasets_dimred[i] = ds[rand_idx, :]
        datasets[i] = datasets[i][rand_idx, :]
        labels += list(np.zeros(datasets_dimred[i].shape[0]) + curr_label)
        curr_label += 1
    labels = np.array(labels, dtype=int)

    embedding = visualize(datasets_dimred,
                          labels, NAMESPACE + '_ds', names,
                          gene_names=mouse_brain_genes, genes=genes,
                          gene_expr=vstack(datasets),
                          multicore_tsne=True,
                          image_suffix='.png')

    cell_labels = (
        open('data/cell_labels/mouse_brain_cluster.txt')
        .read().rstrip().split()
    )
    le = LabelEncoder().fit(cell_labels)
    cell_labels = le.transform(cell_labels)
    cell_types = le.classes_

    visualize(None,
              cell_labels, NAMESPACE + '_type', cell_types,
              embedding=embedding,  image_suffix='.png')



from process import load_names, merge_datasets
from scanorama import *

from time import time
import numpy as np

NAMESPACE = 'panorama'

if __name__ == '__main__':
    from config import data_names

    datasets, genes_list, n_cells = load_names(data_names)

    t0 = time()
    datasets_dimred, datasets, genes = correct(
        datasets, genes_list, ds_names=data_names,
        sigma=150, return_dimred=True
    )
    if VERBOSE:
        print('Integrated and batch corrected panoramas in {:.3f}s'
              .format(time() - t0))

    labels = []
    names = []
    curr_label = 0
    for i, a in enumerate(datasets):
        labels += list(np.zeros(a.shape[0]) + curr_label)
        names.append(data_names[i])
        curr_label += 1
    labels = np.array(labels, dtype=int)
    
    embedding = visualize(datasets_dimred,
                          labels, NAMESPACE + '_ds', names,
                          multicore_tsne=False)

    # Uncorrected.
    datasets, genes_list, n_cells = load_names(data_names)
    datasets, genes = merge_datasets(datasets, genes_list)
    datasets_dimred = dimensionality_reduce(datasets)
    
    labels = []
    names = []
    curr_label = 0
    for i, a in enumerate(datasets):
        labels += list(np.zeros(a.shape[0]) + curr_label)
        names.append(data_names[i])
        curr_label += 1
    labels = np.array(labels, dtype=int)

    embedding = visualize(datasets_dimred, labels,
                          NAMESPACE + '_ds_uncorrected', names)
